/*
    CharityWorld: Agent.h
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

Abstract agent class. The Agent class cannot be used to create actual
agents in the model, but all agents should subclass from this one.

*/

#import <objectbase/SwarmObject.h>
#import <collections/List.h>

@class ModelSwarm, DoubleSimple;

@interface Agent: SwarmObject {
  DoubleSimple *wealth;
  DoubleSimple *localWealth;
  DoubleSimple *minimum;
  DoubleSimple *maximum;
  ModelSwarm *model;
  DoubleSimple *beg;
  DoubleSimple *generosity;
  id <List> nbrs;
  id <List> donatedNbrs;
  int x, y, numberOfNbrs;
  BOOL waiting;
}

+create: (id <Zone>)z;
+create: (id <Zone>)z model: (ModelSwarm *)m X: (int)ax Y: (int)ay;
-(int)getX;
-(int)getY;
-addNeighbour: (Agent *)a;
-addDonatedNeighbour: (Agent *)a;
-clearDonatedNeighbours;
-setBeg: (DoubleSimple *)b;
-setGenerosity: (DoubleSimple *)g;
-(void)computeLocalWealth;
-(BOOL)beggarCondition;
-(BOOL)donorCondition;
-(BOOL)respondingCondition: (Agent *)nbr;
-(BOOL)acceptingCondition: (Agent *)nbr;
-(BOOL)offeringCondition: (Agent *)nbr;
-(BOOL)askingCondition: (Agent *)nbr;
-(BOOL)happinessCondition;
-step;
-ask;
-offer;
-(void)accept: (DoubleSimple *)amount from: (Agent *)a;
-(void)respondTo: (Agent *)a askingFor: (DoubleSimple *)amount;
-(void)addWealth: (DoubleSimple *)amount;
-(void)subWealth: (DoubleSimple *)amount;
-(void)rescheduleCheck;
-(void)reschedule;
-(BOOL)isWaiting;
-(BOOL)donatedTo: (Agent *)a;
-(DoubleSimple *)getWealth;
-(double)getWealthForDisplayOnly;
-(double)getMinimumForDisplayOnly;
-(double)getMaximumForDisplayOnly;
-(double)getBegForDisplayOnly;
-(double)getGenerosityForDisplayOnly;
-drawWealthOn: r;
-drawTracedDonationsOn: r;
-drawLocalWealthOn: r;
-drawFloatingPointClassOn: r;
-drawLog2WealthOn: r;
-drawScheduleOn: r;
-printLocation;
-printWealth;
-(void)drop;

@end
