/*
    CharityWorld: AllSharingSupplyAgents.m
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/* 

This file contains the implementation for two classes:
SharingSupplyAgent and ExtAvgSharingSupplyAgent.
They only differ on the class they inherit from.
The implementation of the methods that they override 
is exactly the same for both classes.
(A dodgy way of doing multiple inheritance )

*/

#import "AllSharingSupplyAgents.h"
#import "DoubleSimple.h"
#import "ModelSwarm.h"
#import "Verbosity.h"


/*

SharingSupplyAgent.m

*/

@implementation SharingSupplyAgent

+create: (id <Zone>)z model: (ModelSwarm *)m X: (int)ax Y: (int)ay {
  
  SharingSupplyAgent *obj;
  obj = [super create: z model: m X: ax Y: ay];

  obj->stepInitialWealth = [DoubleSimple new];
 
  return obj;
}

-(BOOL)donorCondition {
  id <Index> ix = nil;
  Agent *nbr;
  BOOL ans=NO;

  for(ix = [nbrs begin: scratchZone], nbr = (Agent *)[ix next];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix next]) {
    DoubleSimple *nbrWealth = [nbr getWealth];
    if([wealth gt: nbrWealth]) {
      ans = YES;
      [nbrWealth free];
      break;
    }
    [nbrWealth free];
  }
  [ix drop];

  return ans;
}

-(BOOL)offeringCondition: (Agent *)nbr {
  DoubleSimple *nbrWealth = [nbr getWealth];
  BOOL ans;

  ans = [stepInitialWealth gt: nbrWealth];

  [nbrWealth free];

  return ans;
}

-offer {
  id <Index> ix = nil;
  Agent *nbr;
  int nbrPoorer=0;

  [stepInitialWealth set: wealth];

  if([Verbosity showAgentOffering]) {
    printf("Agent (%d, %d) is offering to their neighbours:\n", x + 1, y + 1);
  }

  for(ix = [nbrs begin: scratchZone], nbr = (Agent *)[ix next];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix next]) {
    DoubleSimple *nbrWealth = [nbr getWealth];
    if([wealth gt: nbrWealth]) {
      nbrPoorer++;
    }
    [nbrWealth free];
  }
  [ix drop];

  for(ix = [nbrs begin: scratchZone], nbr = (Agent *)[ix next];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix next]) {
    if([Verbosity showAgentOffering]) {
      printf("\tConsidering neighbour (%d, %d) with wealth ", [nbr getX] + 1,
	     [nbr getY] + 1);
      [[[nbr getWealth] writeAccurately: stdout] free];
      printf("...\n");
    }
    if([self offeringCondition: nbr]) {
      DoubleSimple *genAmount = [stepInitialWealth copy];
      [genAmount idiv: (nbrPoorer+1)];
      if([Verbosity showAgentOffering]) {
	printf("\t...meets offering condition so telling them to accept ");
	[genAmount writeAccurately: stdout];
	printf("\n");
      }
      [nbr accept: genAmount from: self];
      [genAmount free];
    }
    else {
      if([Verbosity showAgentOffering]) {
	printf("\t...does not meet offering condition so ignoring them\n");
      }
    }
  }
  [ix drop];
  for(ix = [nbrs begin: scratchZone], nbr = (Agent *)[ix next];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix next]) {
    if(![nbr isWaiting]) {
      [nbr rescheduleCheck];
    }
  }
  [ix drop];
  return self;
}

-(void)drop {
  if(stepInitialWealth != nil) [stepInitialWealth free];
  [super drop];
}

@end


/*

ExtAvgSharingSupplyAgent.m

*/

@implementation ExtAvgSharingSupplyAgent

+create: (id <Zone>)z model: (ModelSwarm *)m X: (int)ax Y: (int)ay {
  
  ExtAvgSharingSupplyAgent *obj;
  obj = [super create: z model: m X: ax Y: ay];

  obj->stepInitialWealth = [DoubleSimple new];
 
  return obj;
}

-(BOOL)donorCondition {
  id <Index> ix = nil;
  Agent *nbr;
  BOOL ans=NO;

  for(ix = [nbrs begin: scratchZone], nbr = (Agent *)[ix next];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix next]) {
    DoubleSimple *nbrWealth = [nbr getWealth];
    if([wealth gt: nbrWealth]) {
      ans = YES;
      [nbrWealth free];
      break;
    }
    [nbrWealth free];
  }
  [ix drop];

  return ans;
}

-(BOOL)offeringCondition: (Agent *)nbr {
  DoubleSimple *nbrWealth = [nbr getWealth];
  BOOL ans;

  ans = [stepInitialWealth gt: nbrWealth];

  [nbrWealth free];

  return ans;
}

-offer {
  id <Index> ix = nil;
  Agent *nbr;
  int nbrPoorer=0;

  [stepInitialWealth set: wealth];

  if([Verbosity showAgentOffering]) {
    printf("Agent (%d, %d) is offering to their neighbours:\n", x + 1, y + 1);
  }

  for(ix = [nbrs begin: scratchZone], nbr = (Agent *)[ix next];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix next]) {
    DoubleSimple *nbrWealth = [nbr getWealth];
    if([wealth gt: nbrWealth]) {
      nbrPoorer++;
    }
    [nbrWealth free];
  }
  [ix drop];

  for(ix = [nbrs begin: scratchZone], nbr = (Agent *)[ix next];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix next]) {
    if([Verbosity showAgentOffering]) {
      printf("\tConsidering neighbour (%d, %d) with wealth ", [nbr getX] + 1,
	     [nbr getY] + 1);
      [[[nbr getWealth] writeAccurately: stdout] free];
      printf("...\n");
    }
    if([self offeringCondition: nbr]) {
      DoubleSimple *genAmount = [stepInitialWealth copy];
      [genAmount idiv: (nbrPoorer+1)];
      if([Verbosity showAgentOffering]) {
	printf("\t...meets offering condition so telling them to accept ");
	[genAmount writeAccurately: stdout];
	printf("\n");
      }
      [nbr accept: genAmount from: self];
      [genAmount free];
    }
    else {
      if([Verbosity showAgentOffering]) {
	printf("\t...does not meet offering condition so ignoring them\n");
      }
    }
  }
  [ix drop];
  for(ix = [nbrs begin: scratchZone], nbr = (Agent *)[ix next];
      [ix getLoc] == Member;
      nbr = (Agent *)[ix next]) {
    if(![nbr isWaiting]) {
      [nbr rescheduleCheck];
    }
  }
  [ix drop];
  return self;
}

-(void)drop {
  if(stepInitialWealth != nil) [stepInitialWealth free];
  [super drop];
}


@end
