/*
    CharityWorld: DoubleString.m
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

Implementation of DoubleString class

*/

#import "DoubleString.h"
#import <float.h>
#import <stdlib.h>
#import <string.h>
#import <errno.h>
#ifdef INTEL
#  import <CIieeefp.h>
static fp_pctl save_pctl;

#define OPERATOR(method) \
  id retval = [super method: value]; \
  char *buf = (char *)calloc(significant_figures + 10, sizeof(char)); \
  if(buf == NULL) { perror("Memory allocation"); abort(); } \
  save_pctl = fpsetprecision(FP_PC_DBL); \
  sprintf(buf, "%.*e", significant_figures - 1, d); \
  d = atof(buf); \
  fpsetprecision(save_pctl); \
  free(buf); \
  return retval
#else
#define OPERATOR(method) \
  id retval = [super method: value]; \
  char *buf = (char *)calloc(significant_figures + 10, sizeof(char)); \
  if(buf == NULL) { perror("Memory allocation"); abort(); } \
  sprintf(buf, "%.*e", significant_figures - 1, d); \
  d = atof(buf); \
  free(buf); \
  return retval
#endif

/* Class variable */

static int significant_figures = DBL_DIG;

@implementation DoubleString

/* New methods for this class */

+setSignificantFigures: (int)sf {
  if(sf < 1) {
    fprintf(stderr, "Invalid setting for significant figures: %d\n", sf);
    abort();
  }
  if(sf > DBL_DIG) {
    fprintf(stderr, "WARNING: DoubleString used with more significant figures"
	    " than guaranteed by this platform for double precision floating"
	    " point: %d (%d)\n", DBL_DIG, sf);
  }
  significant_figures = sf;
  return self;
}

-setSignificantFigures: (int)sf {
  [[self class] setSignificantFigures: sf];

  return self;
}

+(int)getSignificantFigures {
  return significant_figures;
}

-(int)getSignificantFigures {
  return [[self class] getSignificantFigures];
}

/* Methods to override from DoubleSimple */

+parseValue: (char *)str {
  char *p = strchr(str, (int)'s');

  if(p != NULL) {
    (*p) = '\0';
    p++;
    [self setSignificantFigures: atoi(p)];
  }
  return [super parseValue: str];
}

-add: (DoubleSimple *)value {
  OPERATOR(add);
}

-sub: (DoubleSimple *)value {
  OPERATOR(sub);
}

-mul: (DoubleSimple *)value {
  OPERATOR(mul);
}

-div: (DoubleSimple *)value {
  OPERATOR(div);
}


-iadd: (int)value {
  OPERATOR(iadd);
}

-isub: (int)value {
  OPERATOR(isub);
}

-imul: (int)value {
  OPERATOR(imul);
}

-idiv: (int)value {
  OPERATOR(idiv);
}


-cadd: (double)value {
  OPERATOR(cadd);
}

-csub: (double)value {
  OPERATOR(csub);
}

-cmul: (double)value {
  OPERATOR(cmul);
}

-cdiv: (double)value {
  OPERATOR(cdiv);
}

@end
