/*
    CharityWorld: ModelSwarm.h
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

#import <objectbase/Swarm.h>
#import <activity.h>
#import <collections/List.h>
#import <collections/Array.h>
#import <space/Discrete2d.h>
#import "Queue.h"

@class Agent, DoubleSimple;

@interface ModelSwarm: Swarm {
  id <Queue> beggars;
  id <Queue> donors;
  id <Array> agents;
  id <Schedule> schedule;
  DoubleSimple *generosityMax, *generosityMin;
  char *generosityMaxStr, *generosityMinStr;
  DoubleSimple *begMax, *begMin;
  char *begMaxStr, *begMinStr;
  DoubleSimple *initialWealth;
  char *initialWealthStr;
  int xSize, ySize;
  char *winnerCoord;
  char *loserChooser;
  int radius;
  Discrete2d *world;
  DoubleSimple *jackpot;
  char *jackpotStr;
  char *agentClassStr;
  char *queueClassStr;
  char *envShape;
  char *nbrhood;
  double minWealth, maxWealth;
}

-(void)setGenerosityMinStr: (char *)minStr maxStr: (char *)maxStr;
-(void)setBegMinStr: (char *)minStr maxStr: (char *)maxStr;
-(void)setJackpotStr: (char *)jpStr;
-(void)setInitialWealthStr: (char *)iwStr;
-(void)setAgentClassStr: (char *)acStr;
-(void)setWinnerCoord: (char *)winnerc;
-(void)setLoserChooser: (char *)loserc;
-(void)setQueueClassStr: (char *)qcStr;
-(void)setEnvShape: (char *)shape neighbourhood: (char *)nbr xSize: (int)x
	     ySize: (int)y radius: (int)r;
-buildObjects;
-buildActions;
-next;
-activateIn: (id)swarmContext;
-(void)queueBeggar: (Agent *)a;
-(void)queueDonor: (Agent *)a;
-getWorld;
-getAgents;
-getBeggars;
-getDonors;
-(int)getXSize;
-(int)getYSize;
-(DoubleSimple *)getJackpot;
-(double)getJackpotForDisplayOnly;
-(DoubleSimple *)getInitialWealth;
-(double)getInitialWealthForDisplayOnly;
-(double)getMinWealthForDisplayOnly;
-(double)getMaxWealthForDisplayOnly;
-(void)drop;

@end
