/*
    CharityWorld: PaperModelSwarm.h
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

#import <objectbase/Swarm.h>

#import "ModelSwarm.h"

@interface PaperModelSwarm: Swarm {
  ModelSwarm *model;

  char *generosityStr;
  char *initialWealthStr;
  int xSize, ySize;
  char *winnerCoord;
  char *loserChooser;
  char *jackpotStr;
  char *agentClassStr;
}

-buildObjects;

-buildActions;			// Although we're forwarding, all Swarm 
				// class methods over-ridden by ModelSwarm
                                // need to be over-ridden here
-activateIn: swarmContext;	// ditto
-(void)drop;			// and again.

-(retval_t)forward: (SEL)sel : (arglist_t)args;

/* Since retval_t is a pointer, any ModelSwarm methods that return a
 * non-pointer (e.g. double, int) need to be re-implemented to prevent
 * a crash. int is probably OK, of course, but better not to rely on
 * it.
 */

-(int)getXSize;
-(int)getYSize;

-(double)getJackpotForDisplayOnly;
-(double)getInitialWealthForDisplayOnly;
-(double)getMinWealthForDisplayOnly;
-(double)getMaxWealthForDisplayOnly;

/* These methods should not really need to be re-implemented, but
 * Swarm 2.1.1 cannot handle message forwarding, and I have my
 * suspicions about Swarm 2.2.
 */

-(void)queueBeggar: (Agent *)a;
-(void)queueDonor: (Agent *)a;
-getWorld;
-getAgents;
-getBeggars;
-getDonors;
-(DoubleSimple *)getJackpot;
-(DoubleSimple *)getInitialWealth;

@end
