/*
    CharityWorld: Shuffler.m
    Copyright (C) 2003  Macaulay Institute

    This file is part of CharityWorld, a simple agent-based model to
    demonstrate the effects of errors in floating point arithmetic on
    emergent outcomes.

    CharityWorld is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CharityWorld is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/

#import "Shuffler.h"
#import "ShuffleTuple.h"
#import "MergeSorter.h"
#import <random.h>
#import <objectbase/SwarmObject.h>
#import <defobj/Zone.h>


@implementation Shuffler

+shuffleList: (id <List>)list {
  id <Array> temp;
  id <Zone> zone;
  int i, c;

  c = [list getCount];
  zone = [Zone create: scratchZone];
  temp = [Array create: zone setCount: c];

  for(i = 0; i < c; i++) {
    ShuffleTuple *t
      = [ShuffleTuple create: zone
		      setObj: [list removeFirst]
		      setNum: [uniformDblRand
				getDoubleWithMin: 0.0
				withMax: 1.0]];
    [temp atOffset: i put: t];
  }
  [MergeSorter mergeSort: temp withSelector: M(getNum)];
  for(i = 0; i < c; i++) {
    [list addFirst: (ShuffleTuple *)[[temp atOffset: i] getObj]];
  }
  [zone drop];
  return list;
}

+shuffleArray: (id <Array>)array {
  id <Array> temp;
  id <Zone> zone;
  int i, c;

  c = [array getCount];
  zone = [Zone create: scratchZone];
  temp = [Array create: zone setCount: c];

  for(i = 0; i < c; i++) {
    ShuffleTuple *t
      = [ShuffleTuple create: zone
		      setObj: [array atOffset: i]
		      setNum: [uniformDblRand
				getDoubleWithMin: 0.0
				withMax: 1.0]];
    [temp atOffset: i put: t];
  }
  [MergeSorter mergeSort: temp withSelector: M(getNum)];
  for(i = 0; i < c; i++) {
    [array atOffset: i put: [[temp atOffset: i] getObj]];
  }
  [zone drop];
  return array;
}

@end
